@@ -1,4 +1,4 @@
-/* $Id: gucharmap-mini-fontsel.c,v 1.3 2003/06/01 07:06:49 nlevitt Exp $ */
+/* $Id: gucharmap-mini-fontsel.c,v 1.5 2003/09/04 22:39:16 nlevitt Exp $ */
 /*
  * Copyright (c) 2003 Noah Levitt
  *
@@ -90,6 +90,7 @@ show_available_families (GucharmapMiniFontSelection *fontsel)
   gtk_combo_set_popdown_strings (GTK_COMBO (fontsel->family), family_names);
     
   g_list_free (family_names);
+  g_free (families);
 }
 
 
@@ -140,6 +141,13 @@ size_changed (GtkAdjustment *adjustment,
 }
 
 
+static void
+mini_font_selection_finalize (GucharmapMiniFontSelection *fontsel)
+{
+  pango_font_description_free (fontsel->font_desc);
+}
+
+
 void
 gucharmap_mini_font_selection_class_init (GucharmapMiniFontSelectionClass *clazz)
 {
@@ -151,6 +159,8 @@ gucharmap_mini_font_selection_class_init (GucharmapMiniFontSelectionClass *clazz
                     G_STRUCT_OFFSET (GucharmapMiniFontSelectionClass, changed),
                     NULL, NULL, gucharmap_marshal_VOID__VOID,
                     G_TYPE_NONE, 0);
+
+  G_OBJECT_CLASS (clazz)->finalize = mini_font_selection_finalize;
 }
 
 
