@@ -1,4 +1,4 @@
-/* $Id: gucharmap-window.c,v 1.19 2003/07/07 20:23:36 nlevitt Exp $ */
+/* $Id: gucharmap-window.c,v 1.20 2003/09/04 22:39:16 nlevitt Exp $ */
 /*
  * Copyright (c) 2003  Noah Levitt <nlevitt аt columbia.edu>
  *
@@ -1004,10 +1004,27 @@ show_all (GtkWidget *widget)
 }
 
 
+static void
+window_finalize (GucharmapWindow *guw)
+{
+  if (guw->last_search)
+    g_free (guw->last_search);
+
+  if (guw->history)
+    {
+      while (guw->history->prev)
+        guw->history = guw->history->prev;
+
+      g_list_free (guw->history);
+    }
+}
+
+
 static void
 gucharmap_window_class_init (GucharmapWindowClass *clazz)
 {
   GTK_WIDGET_CLASS (clazz)->show_all = show_all;
+  G_OBJECT_CLASS (clazz)->finalize = window_finalize;
 }
 
 
